
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <fstream>
#include <sstream>

#include "TString.h"
#include "TRandom.h"
#include "TMath.h"
#include "TH1F.h"
#include "TF1.h"
#include "TCanvas.h"
#include "TFile.h"
#include "TLine.h"
#include "TGraph.h"
#include "TClonesArray.h"
#include "THStack.h"
#include "TLegend.h"
#include "TLimitDataSource.h"
#include "TConfidenceLevel.h"
#include "TLimit.h"
#include "TGraph2D.h"
#include "TPolyLine.h"
#include "TPaveText.h"
#include "TAxis.h"
#include "TH2F.h"

#include "RooRealVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooConstVar.h"
#include "RooPolynomial.h"
#include "RooHistPdf.h"
#include "RooPlot.h"
#include "RooDataHist.h"
#include "RooAddPdf.h"
#include "RooArgSet.h"
#include "RooExtendPdf.h"
#include "RooConstVar.h"
#include "RooRandom.h"

#include "RooStats/HybridCalculatorOriginal.h"
#include "RooStats/HybridPlot.h"
#include "RooStats/HybridResult.h"


using namespace RooFit;
using namespace RooStats;
using namespace std;

// ********************************************************************

void getCLs(const char* obs_name, TH1F* sig, TH1F* bkg, float yieldSig, float yieldBkg, int ntoys);

// ********************************************************************

int main( int argc, const char* argv[] )
{


	TFile* fileSig;
	TFile* fileBkg;
	
	TH1F* distSig;
	TH1F* distBkg;

	// Read histo from files

	fileSig = new TFile("inputDistSig.root","READ");
	distSig = (TH1F*) fileSig->Get("MET");

	fileBkg = new TFile("inputDistBkg.root","READ");
	distBkg = (TH1F*) fileBkg->Get("MET");

	TFile* outputSig = new TFile("outputSig.root","RECREATE");
	distSig->Write();
	TFile* outputBkg = new TFile("outputBkg.root","RECREATE");
	distBkg->Write();

	
	// Call function computing CLs

	getCLs("MET",distSig,distBkg,50,50000,50000);
	
	fileSig->Close();
	fileBkg->Close();
	
}


void getCLs(const char* obs_name, TH1F* sig, TH1F* bkg, float yieldSig_input, float yieldBkg_input, int ntoys)
{

	// ************************************
	// Define histo, pdf, parameters...
	// ************************************

	RooRandom::randomGenerator()->SetSeed();

	// Get range and nbins
	
	int obs_nbins = bkg->GetNbinsX();
	float obs_min = bkg->GetXaxis()->GetXmin();
	float obs_max = bkg->GetXaxis()->GetXmax();

	// Define observable

    RooRealVar obs(obs_name,obs_name,obs_min,obs_max) ;
  	obs.setBins(obs_nbins);

	// Convert TH1F into PDF

	RooDataHist* rooHistBkg = new RooDataHist("rooHistBkg","Background RooFit/RooStat histogram",obs,bkg);
	RooHistPdf pdfBkg("pdfBkg","Background pdf",obs,*rooHistBkg,0);

	RooDataHist* rooHistSig = new RooDataHist("rooHistSig","Signal RooFit/RooStat histogram",obs,sig);
	RooHistPdf pdfSig("pdfSig","Signal pdf",obs,*rooHistSig,0);

	// Set yields

	RooRealVar yieldSig("yieldSig","",yieldSig_input,0,10000);
	RooRealVar yieldBkg("yieldBkg","",yieldBkg_input,0,100000000);
	
	RooExtendPdf pdfBkgExtended("pdfBkgExtended","",pdfBkg,yieldBkg);

	// Compute sig + bkg pdf or only bkg
	
	RooAddPdf pdfTot("pdfTot","",RooArgList(pdfSig,pdfBkg),RooArgList(yieldSig,yieldBkg));

	// ************************************
	// Run HybridCalculator on those inputs
	// ************************************
	
			//!! (no nuisance parameters taken into account for now)
		
		HybridCalculatorOriginal myHybridCalc(*rooHistBkg, pdfTot, pdfBkgExtended);
		myHybridCalc.UseNuisance(false);

			// use test statitics 2*ln(Q) with N toy-experiments
		
		myHybridCalc.SetTestStatistic(1);
		myHybridCalc.SetNumberOfToys(ntoys);

			//!! Not sure what this do. From tutorial :
			// "For speed up generation (do binned data) :"
			// myHybridCalc.SetGenerateBinned(false);

			// (putting false slow down the calculation like hell)

		myHybridCalc.SetGenerateBinned(true);

			// Compute test
			
		HybridResult* myHybridResult = myHybridCalc.GetHypoTest();
		if (!myHybridResult)
		{
			cerr << "\n Error from Hypothesis test" << endl;
			return;
		}

	// ************************************
	// Print / plot results
	// ************************************

	// Make plot 
	HybridPlot* myHybridPlot = myHybridResult->GetPlot("myHybridPlot","Plot of results with HybridCalculatorOriginal",50);
	myHybridResult->SetDataTestStatistics(myHybridPlot->GetBmean());
	myHybridPlot = myHybridResult->GetPlot("myHybridPlot","Plot of results with HybridCalculatorOriginal",50);

	// Save plot
	TFile* output = new TFile("output.root","RECREATE");
	TCanvas* canvas = new TCanvas("canvas","canvas",800,600);	
		
	myHybridPlot->Draw();
	canvas->Write();



	// From plotn, get mean +/- 1sig of B and S+B p-value distribution
	float Bmean = myHybridPlot->GetBmean();
	float Brms = myHybridPlot->GetBrms();
	float Bmean_p1sig = Bmean + Brms;
	float Bmean_m1sig = Bmean - Brms;

	// Print results
	// and save it in output_tab
	myHybridResult->SetDataTestStatistics(Bmean);
	cout << "[    Expected  CLs    ]  =  " << myHybridResult->CLs() << endl;
	
	myHybridResult->SetDataTestStatistics(Bmean_p1sig);
	cout << "[ Expected CLs + 1sig ] = " << myHybridResult->CLs() << endl;

	myHybridResult->SetDataTestStatistics(Bmean_m1sig);
	cout << "[ Expected CLs - 1sig ] = " << myHybridResult->CLs() << endl;
	

	return;
}


